# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: t_protocol_no_ct.proto

import sys

_b = sys.version_info[0] < 3 and (lambda x: x) or (lambda x: x.encode("latin1"))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor.FileDescriptor(
    name="t_protocol_no_ct.proto",
    package="aea.fetchai.t_protocol_no_ct",
    syntax="proto3",
    serialized_options=None,
    serialized_pb=_b(
        '\n\x16t_protocol_no_ct.proto\x12\x1c\x61\x65\x61.fetchai.t_protocol_no_ct"\x84\x30\n\x14TProtocolNoCtMessage\x12\x82\x01\n\x1bperformative_empty_contents\x18\x05 \x01(\x0b\x32[.aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Empty_Contents_PerformativeH\x00\x12j\n\x0fperformative_mt\x18\x06 \x01(\x0b\x32O.aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Mt_PerformativeH\x00\x12h\n\x0eperformative_o\x18\x07 \x01(\x0b\x32N.aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_O_PerformativeH\x00\x12l\n\x10performative_pct\x18\x08 \x01(\x0b\x32P.aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pct_PerformativeH\x00\x12l\n\x10performative_pmt\x18\t \x01(\x0b\x32P.aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_PerformativeH\x00\x12j\n\x0fperformative_pt\x18\n \x01(\x0b\x32O.aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pt_PerformativeH\x00\x1a\x8c\x01\n\x1cPerformative_Pt_Performative\x12\x15\n\rcontent_bytes\x18\x01 \x01(\x0c\x12\x13\n\x0b\x63ontent_int\x18\x02 \x01(\x05\x12\x15\n\rcontent_float\x18\x03 \x01(\x02\x12\x14\n\x0c\x63ontent_bool\x18\x04 \x01(\x08\x12\x13\n\x0b\x63ontent_str\x18\x05 \x01(\t\x1a\xa8\x02\n\x1dPerformative_Pct_Performative\x12\x19\n\x11\x63ontent_set_bytes\x18\x01 \x03(\x0c\x12\x17\n\x0f\x63ontent_set_int\x18\x02 \x03(\x05\x12\x19\n\x11\x63ontent_set_float\x18\x03 \x03(\x02\x12\x18\n\x10\x63ontent_set_bool\x18\x04 \x03(\x08\x12\x17\n\x0f\x63ontent_set_str\x18\x05 \x03(\t\x12\x1a\n\x12\x63ontent_list_bytes\x18\x06 \x03(\x0c\x12\x18\n\x10\x63ontent_list_int\x18\x07 \x03(\x05\x12\x1a\n\x12\x63ontent_list_float\x18\x08 \x03(\x02\x12\x19\n\x11\x63ontent_list_bool\x18\t \x03(\x08\x12\x18\n\x10\x63ontent_list_str\x18\n \x03(\t\x1a\xb9\x17\n\x1dPerformative_Pmt_Performative\x12\x89\x01\n\x16\x63ontent_dict_int_bytes\x18\x01 \x03(\x0b\x32i.aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictIntBytesEntry\x12\x85\x01\n\x14\x63ontent_dict_int_int\x18\x02 \x03(\x0b\x32g.aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictIntIntEntry\x12\x89\x01\n\x16\x63ontent_dict_int_float\x18\x03 \x03(\x0b\x32i.aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictIntFloatEntry\x12\x87\x01\n\x15\x63ontent_dict_int_bool\x18\x04 \x03(\x0b\x32h.aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictIntBoolEntry\x12\x85\x01\n\x14\x63ontent_dict_int_str\x18\x05 \x03(\x0b\x32g.aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictIntStrEntry\x12\x8b\x01\n\x17\x63ontent_dict_bool_bytes\x18\x06 \x03(\x0b\x32j.aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictBoolBytesEntry\x12\x87\x01\n\x15\x63ontent_dict_bool_int\x18\x07 \x03(\x0b\x32h.aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictBoolIntEntry\x12\x8b\x01\n\x17\x63ontent_dict_bool_float\x18\x08 \x03(\x0b\x32j.aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictBoolFloatEntry\x12\x89\x01\n\x16\x63ontent_dict_bool_bool\x18\t \x03(\x0b\x32i.aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictBoolBoolEntry\x12\x87\x01\n\x15\x63ontent_dict_bool_str\x18\n \x03(\x0b\x32h.aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictBoolStrEntry\x12\x89\x01\n\x16\x63ontent_dict_str_bytes\x18\x0b \x03(\x0b\x32i.aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictStrBytesEntry\x12\x85\x01\n\x14\x63ontent_dict_str_int\x18\x0c \x03(\x0b\x32g.aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictStrIntEntry\x12\x89\x01\n\x16\x63ontent_dict_str_float\x18\r \x03(\x0b\x32i.aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictStrFloatEntry\x12\x87\x01\n\x15\x63ontent_dict_str_bool\x18\x0e \x03(\x0b\x32h.aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictStrBoolEntry\x12\x85\x01\n\x14\x63ontent_dict_str_str\x18\x0f \x03(\x0b\x32g.aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictStrStrEntry\x1a:\n\x18\x43ontentDictIntBytesEntry\x12\x0b\n\x03key\x18\x01 \x01(\x05\x12\r\n\x05value\x18\x02 \x01(\x0c:\x02\x38\x01\x1a\x38\n\x16\x43ontentDictIntIntEntry\x12\x0b\n\x03key\x18\x01 \x01(\x05\x12\r\n\x05value\x18\x02 \x01(\x05:\x02\x38\x01\x1a:\n\x18\x43ontentDictIntFloatEntry\x12\x0b\n\x03key\x18\x01 \x01(\x05\x12\r\n\x05value\x18\x02 \x01(\x02:\x02\x38\x01\x1a\x39\n\x17\x43ontentDictIntBoolEntry\x12\x0b\n\x03key\x18\x01 \x01(\x05\x12\r\n\x05value\x18\x02 \x01(\x08:\x02\x38\x01\x1a\x38\n\x16\x43ontentDictIntStrEntry\x12\x0b\n\x03key\x18\x01 \x01(\x05\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a;\n\x19\x43ontentDictBoolBytesEntry\x12\x0b\n\x03key\x18\x01 \x01(\x08\x12\r\n\x05value\x18\x02 \x01(\x0c:\x02\x38\x01\x1a\x39\n\x17\x43ontentDictBoolIntEntry\x12\x0b\n\x03key\x18\x01 \x01(\x08\x12\r\n\x05value\x18\x02 \x01(\x05:\x02\x38\x01\x1a;\n\x19\x43ontentDictBoolFloatEntry\x12\x0b\n\x03key\x18\x01 \x01(\x08\x12\r\n\x05value\x18\x02 \x01(\x02:\x02\x38\x01\x1a:\n\x18\x43ontentDictBoolBoolEntry\x12\x0b\n\x03key\x18\x01 \x01(\x08\x12\r\n\x05value\x18\x02 \x01(\x08:\x02\x38\x01\x1a\x39\n\x17\x43ontentDictBoolStrEntry\x12\x0b\n\x03key\x18\x01 \x01(\x08\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a:\n\x18\x43ontentDictStrBytesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x0c:\x02\x38\x01\x1a\x38\n\x16\x43ontentDictStrIntEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x05:\x02\x38\x01\x1a:\n\x18\x43ontentDictStrFloatEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x02:\x02\x38\x01\x1a\x39\n\x17\x43ontentDictStrBoolEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x08:\x02\x38\x01\x1a\x38\n\x16\x43ontentDictStrStrEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\xd4\x0b\n\x1cPerformative_Mt_Performative\x12"\n\x1a\x63ontent_union_1_type_bytes\x18\x01 \x01(\x0c\x12 \n\x18\x63ontent_union_1_type_int\x18\x02 \x01(\x05\x12"\n\x1a\x63ontent_union_1_type_float\x18\x03 \x01(\x02\x12!\n\x19\x63ontent_union_1_type_bool\x18\x04 \x01(\x08\x12 \n\x18\x63ontent_union_1_type_str\x18\x05 \x01(\t\x12\'\n\x1f\x63ontent_union_1_type_set_of_int\x18\x06 \x03(\x05\x12)\n!content_union_1_type_list_of_bool\x18\x07 \x03(\x08\x12\xa0\x01\n$content_union_1_type_dict_of_str_int\x18\x08 \x03(\x0b\x32r.aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Mt_Performative.ContentUnion1TypeDictOfStrIntEntry\x12)\n!content_union_2_type_set_of_bytes\x18\t \x03(\x0c\x12\'\n\x1f\x63ontent_union_2_type_set_of_int\x18\n \x03(\x05\x12\'\n\x1f\x63ontent_union_2_type_set_of_str\x18\x0b \x03(\t\x12*\n"content_union_2_type_list_of_float\x18\x0c \x03(\x02\x12)\n!content_union_2_type_list_of_bool\x18\r \x03(\x08\x12*\n"content_union_2_type_list_of_bytes\x18\x0e \x03(\x0c\x12\xa0\x01\n$content_union_2_type_dict_of_str_int\x18\x0f \x03(\x0b\x32r.aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Mt_Performative.ContentUnion2TypeDictOfStrIntEntry\x12\xa4\x01\n&content_union_2_type_dict_of_int_float\x18\x10 \x03(\x0b\x32t.aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Mt_Performative.ContentUnion2TypeDictOfIntFloatEntry\x12\xa6\x01\n\'content_union_2_type_dict_of_bool_bytes\x18\x11 \x03(\x0b\x32u.aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Mt_Performative.ContentUnion2TypeDictOfBoolBytesEntry\x1a\x44\n"ContentUnion1TypeDictOfStrIntEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x05:\x02\x38\x01\x1a\x44\n"ContentUnion2TypeDictOfStrIntEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x05:\x02\x38\x01\x1a\x46\n$ContentUnion2TypeDictOfIntFloatEntry\x12\x0b\n\x03key\x18\x01 \x01(\x05\x12\r\n\x05value\x18\x02 \x01(\x02:\x02\x38\x01\x1aG\n%ContentUnion2TypeDictOfBoolBytesEntry\x12\x0b\n\x03key\x18\x01 \x01(\x08\x12\r\n\x05value\x18\x02 \x01(\x0c:\x02\x38\x01\x1a\xbf\x03\n\x1bPerformative_O_Performative\x12\x16\n\x0e\x63ontent_o_bool\x18\x01 \x01(\x08\x12\x1d\n\x15\x63ontent_o_bool_is_set\x18\x02 \x01(\x08\x12\x19\n\x11\x63ontent_o_set_int\x18\x03 \x03(\x05\x12 \n\x18\x63ontent_o_set_int_is_set\x18\x04 \x01(\x08\x12\x1c\n\x14\x63ontent_o_list_bytes\x18\x05 \x03(\x0c\x12#\n\x1b\x63ontent_o_list_bytes_is_set\x18\x06 \x01(\x08\x12\x86\x01\n\x16\x63ontent_o_dict_str_int\x18\x07 \x03(\x0b\x32\x66.aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_O_Performative.ContentODictStrIntEntry\x12%\n\x1d\x63ontent_o_dict_str_int_is_set\x18\x08 \x01(\x08\x1a\x39\n\x17\x43ontentODictStrIntEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x05:\x02\x38\x01\x1a*\n(Performative_Empty_Contents_PerformativeB\x0e\n\x0cperformativeb\x06proto3'
    ),
)


_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PT_PERFORMATIVE = _descriptor.Descriptor(
    name="Performative_Pt_Performative",
    full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pt_Performative",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="content_bytes",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pt_Performative.content_bytes",
            index=0,
            number=1,
            type=12,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b(""),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="content_int",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pt_Performative.content_int",
            index=1,
            number=2,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="content_float",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pt_Performative.content_float",
            index=2,
            number=3,
            type=2,
            cpp_type=6,
            label=1,
            has_default_value=False,
            default_value=float(0),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="content_bool",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pt_Performative.content_bool",
            index=3,
            number=4,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="content_str",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pt_Performative.content_str",
            index=4,
            number=5,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=757,
    serialized_end=897,
)

_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PCT_PERFORMATIVE = _descriptor.Descriptor(
    name="Performative_Pct_Performative",
    full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pct_Performative",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="content_set_bytes",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pct_Performative.content_set_bytes",
            index=0,
            number=1,
            type=12,
            cpp_type=9,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="content_set_int",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pct_Performative.content_set_int",
            index=1,
            number=2,
            type=5,
            cpp_type=1,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="content_set_float",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pct_Performative.content_set_float",
            index=2,
            number=3,
            type=2,
            cpp_type=6,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="content_set_bool",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pct_Performative.content_set_bool",
            index=3,
            number=4,
            type=8,
            cpp_type=7,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="content_set_str",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pct_Performative.content_set_str",
            index=4,
            number=5,
            type=9,
            cpp_type=9,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="content_list_bytes",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pct_Performative.content_list_bytes",
            index=5,
            number=6,
            type=12,
            cpp_type=9,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="content_list_int",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pct_Performative.content_list_int",
            index=6,
            number=7,
            type=5,
            cpp_type=1,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="content_list_float",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pct_Performative.content_list_float",
            index=7,
            number=8,
            type=2,
            cpp_type=6,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="content_list_bool",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pct_Performative.content_list_bool",
            index=8,
            number=9,
            type=8,
            cpp_type=7,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="content_list_str",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pct_Performative.content_list_str",
            index=9,
            number=10,
            type=9,
            cpp_type=9,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=900,
    serialized_end=1196,
)

_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTINTBYTESENTRY = _descriptor.Descriptor(
    name="ContentDictIntBytesEntry",
    full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictIntBytesEntry",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="key",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictIntBytesEntry.key",
            index=0,
            number=1,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="value",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictIntBytesEntry.value",
            index=1,
            number=2,
            type=12,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b(""),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=_b("8\001"),
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=3312,
    serialized_end=3370,
)

_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTINTINTENTRY = _descriptor.Descriptor(
    name="ContentDictIntIntEntry",
    full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictIntIntEntry",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="key",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictIntIntEntry.key",
            index=0,
            number=1,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="value",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictIntIntEntry.value",
            index=1,
            number=2,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=_b("8\001"),
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=3372,
    serialized_end=3428,
)

_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTINTFLOATENTRY = _descriptor.Descriptor(
    name="ContentDictIntFloatEntry",
    full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictIntFloatEntry",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="key",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictIntFloatEntry.key",
            index=0,
            number=1,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="value",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictIntFloatEntry.value",
            index=1,
            number=2,
            type=2,
            cpp_type=6,
            label=1,
            has_default_value=False,
            default_value=float(0),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=_b("8\001"),
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=3430,
    serialized_end=3488,
)

_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTINTBOOLENTRY = _descriptor.Descriptor(
    name="ContentDictIntBoolEntry",
    full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictIntBoolEntry",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="key",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictIntBoolEntry.key",
            index=0,
            number=1,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="value",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictIntBoolEntry.value",
            index=1,
            number=2,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=_b("8\001"),
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=3490,
    serialized_end=3547,
)

_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTINTSTRENTRY = _descriptor.Descriptor(
    name="ContentDictIntStrEntry",
    full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictIntStrEntry",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="key",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictIntStrEntry.key",
            index=0,
            number=1,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="value",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictIntStrEntry.value",
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=_b("8\001"),
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=3549,
    serialized_end=3605,
)

_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTBOOLBYTESENTRY = _descriptor.Descriptor(
    name="ContentDictBoolBytesEntry",
    full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictBoolBytesEntry",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="key",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictBoolBytesEntry.key",
            index=0,
            number=1,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="value",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictBoolBytesEntry.value",
            index=1,
            number=2,
            type=12,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b(""),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=_b("8\001"),
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=3607,
    serialized_end=3666,
)

_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTBOOLINTENTRY = _descriptor.Descriptor(
    name="ContentDictBoolIntEntry",
    full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictBoolIntEntry",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="key",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictBoolIntEntry.key",
            index=0,
            number=1,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="value",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictBoolIntEntry.value",
            index=1,
            number=2,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=_b("8\001"),
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=3668,
    serialized_end=3725,
)

_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTBOOLFLOATENTRY = _descriptor.Descriptor(
    name="ContentDictBoolFloatEntry",
    full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictBoolFloatEntry",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="key",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictBoolFloatEntry.key",
            index=0,
            number=1,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="value",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictBoolFloatEntry.value",
            index=1,
            number=2,
            type=2,
            cpp_type=6,
            label=1,
            has_default_value=False,
            default_value=float(0),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=_b("8\001"),
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=3727,
    serialized_end=3786,
)

_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTBOOLBOOLENTRY = _descriptor.Descriptor(
    name="ContentDictBoolBoolEntry",
    full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictBoolBoolEntry",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="key",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictBoolBoolEntry.key",
            index=0,
            number=1,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="value",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictBoolBoolEntry.value",
            index=1,
            number=2,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=_b("8\001"),
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=3788,
    serialized_end=3846,
)

_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTBOOLSTRENTRY = _descriptor.Descriptor(
    name="ContentDictBoolStrEntry",
    full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictBoolStrEntry",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="key",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictBoolStrEntry.key",
            index=0,
            number=1,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="value",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictBoolStrEntry.value",
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=_b("8\001"),
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=3848,
    serialized_end=3905,
)

_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTSTRBYTESENTRY = _descriptor.Descriptor(
    name="ContentDictStrBytesEntry",
    full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictStrBytesEntry",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="key",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictStrBytesEntry.key",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="value",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictStrBytesEntry.value",
            index=1,
            number=2,
            type=12,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b(""),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=_b("8\001"),
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=3907,
    serialized_end=3965,
)

_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTSTRINTENTRY = _descriptor.Descriptor(
    name="ContentDictStrIntEntry",
    full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictStrIntEntry",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="key",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictStrIntEntry.key",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="value",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictStrIntEntry.value",
            index=1,
            number=2,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=_b("8\001"),
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=3967,
    serialized_end=4023,
)

_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTSTRFLOATENTRY = _descriptor.Descriptor(
    name="ContentDictStrFloatEntry",
    full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictStrFloatEntry",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="key",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictStrFloatEntry.key",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="value",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictStrFloatEntry.value",
            index=1,
            number=2,
            type=2,
            cpp_type=6,
            label=1,
            has_default_value=False,
            default_value=float(0),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=_b("8\001"),
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=4025,
    serialized_end=4083,
)

_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTSTRBOOLENTRY = _descriptor.Descriptor(
    name="ContentDictStrBoolEntry",
    full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictStrBoolEntry",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="key",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictStrBoolEntry.key",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="value",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictStrBoolEntry.value",
            index=1,
            number=2,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=_b("8\001"),
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=4085,
    serialized_end=4142,
)

_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTSTRSTRENTRY = _descriptor.Descriptor(
    name="ContentDictStrStrEntry",
    full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictStrStrEntry",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="key",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictStrStrEntry.key",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="value",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictStrStrEntry.value",
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=_b("8\001"),
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=4144,
    serialized_end=4200,
)

_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE = _descriptor.Descriptor(
    name="Performative_Pmt_Performative",
    full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="content_dict_int_bytes",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.content_dict_int_bytes",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="content_dict_int_int",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.content_dict_int_int",
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="content_dict_int_float",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.content_dict_int_float",
            index=2,
            number=3,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="content_dict_int_bool",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.content_dict_int_bool",
            index=3,
            number=4,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="content_dict_int_str",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.content_dict_int_str",
            index=4,
            number=5,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="content_dict_bool_bytes",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.content_dict_bool_bytes",
            index=5,
            number=6,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="content_dict_bool_int",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.content_dict_bool_int",
            index=6,
            number=7,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="content_dict_bool_float",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.content_dict_bool_float",
            index=7,
            number=8,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="content_dict_bool_bool",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.content_dict_bool_bool",
            index=8,
            number=9,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="content_dict_bool_str",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.content_dict_bool_str",
            index=9,
            number=10,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="content_dict_str_bytes",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.content_dict_str_bytes",
            index=10,
            number=11,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="content_dict_str_int",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.content_dict_str_int",
            index=11,
            number=12,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="content_dict_str_float",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.content_dict_str_float",
            index=12,
            number=13,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="content_dict_str_bool",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.content_dict_str_bool",
            index=13,
            number=14,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="content_dict_str_str",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.content_dict_str_str",
            index=14,
            number=15,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[
        _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTINTBYTESENTRY,
        _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTINTINTENTRY,
        _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTINTFLOATENTRY,
        _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTINTBOOLENTRY,
        _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTINTSTRENTRY,
        _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTBOOLBYTESENTRY,
        _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTBOOLINTENTRY,
        _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTBOOLFLOATENTRY,
        _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTBOOLBOOLENTRY,
        _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTBOOLSTRENTRY,
        _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTSTRBYTESENTRY,
        _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTSTRINTENTRY,
        _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTSTRFLOATENTRY,
        _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTSTRBOOLENTRY,
        _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTSTRSTRENTRY,
    ],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=1199,
    serialized_end=4200,
)

_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_MT_PERFORMATIVE_CONTENTUNION1TYPEDICTOFSTRINTENTRY = _descriptor.Descriptor(
    name="ContentUnion1TypeDictOfStrIntEntry",
    full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Mt_Performative.ContentUnion1TypeDictOfStrIntEntry",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="key",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Mt_Performative.ContentUnion1TypeDictOfStrIntEntry.key",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="value",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Mt_Performative.ContentUnion1TypeDictOfStrIntEntry.value",
            index=1,
            number=2,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=_b("8\001"),
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=5412,
    serialized_end=5480,
)

_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_MT_PERFORMATIVE_CONTENTUNION2TYPEDICTOFSTRINTENTRY = _descriptor.Descriptor(
    name="ContentUnion2TypeDictOfStrIntEntry",
    full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Mt_Performative.ContentUnion2TypeDictOfStrIntEntry",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="key",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Mt_Performative.ContentUnion2TypeDictOfStrIntEntry.key",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="value",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Mt_Performative.ContentUnion2TypeDictOfStrIntEntry.value",
            index=1,
            number=2,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=_b("8\001"),
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=5482,
    serialized_end=5550,
)

_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_MT_PERFORMATIVE_CONTENTUNION2TYPEDICTOFINTFLOATENTRY = _descriptor.Descriptor(
    name="ContentUnion2TypeDictOfIntFloatEntry",
    full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Mt_Performative.ContentUnion2TypeDictOfIntFloatEntry",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="key",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Mt_Performative.ContentUnion2TypeDictOfIntFloatEntry.key",
            index=0,
            number=1,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="value",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Mt_Performative.ContentUnion2TypeDictOfIntFloatEntry.value",
            index=1,
            number=2,
            type=2,
            cpp_type=6,
            label=1,
            has_default_value=False,
            default_value=float(0),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=_b("8\001"),
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=5552,
    serialized_end=5622,
)

_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_MT_PERFORMATIVE_CONTENTUNION2TYPEDICTOFBOOLBYTESENTRY = _descriptor.Descriptor(
    name="ContentUnion2TypeDictOfBoolBytesEntry",
    full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Mt_Performative.ContentUnion2TypeDictOfBoolBytesEntry",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="key",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Mt_Performative.ContentUnion2TypeDictOfBoolBytesEntry.key",
            index=0,
            number=1,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="value",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Mt_Performative.ContentUnion2TypeDictOfBoolBytesEntry.value",
            index=1,
            number=2,
            type=12,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b(""),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=_b("8\001"),
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=5624,
    serialized_end=5695,
)

_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_MT_PERFORMATIVE = _descriptor.Descriptor(
    name="Performative_Mt_Performative",
    full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Mt_Performative",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="content_union_1_type_bytes",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Mt_Performative.content_union_1_type_bytes",
            index=0,
            number=1,
            type=12,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b(""),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="content_union_1_type_int",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Mt_Performative.content_union_1_type_int",
            index=1,
            number=2,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="content_union_1_type_float",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Mt_Performative.content_union_1_type_float",
            index=2,
            number=3,
            type=2,
            cpp_type=6,
            label=1,
            has_default_value=False,
            default_value=float(0),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="content_union_1_type_bool",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Mt_Performative.content_union_1_type_bool",
            index=3,
            number=4,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="content_union_1_type_str",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Mt_Performative.content_union_1_type_str",
            index=4,
            number=5,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="content_union_1_type_set_of_int",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Mt_Performative.content_union_1_type_set_of_int",
            index=5,
            number=6,
            type=5,
            cpp_type=1,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="content_union_1_type_list_of_bool",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Mt_Performative.content_union_1_type_list_of_bool",
            index=6,
            number=7,
            type=8,
            cpp_type=7,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="content_union_1_type_dict_of_str_int",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Mt_Performative.content_union_1_type_dict_of_str_int",
            index=7,
            number=8,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="content_union_2_type_set_of_bytes",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Mt_Performative.content_union_2_type_set_of_bytes",
            index=8,
            number=9,
            type=12,
            cpp_type=9,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="content_union_2_type_set_of_int",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Mt_Performative.content_union_2_type_set_of_int",
            index=9,
            number=10,
            type=5,
            cpp_type=1,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="content_union_2_type_set_of_str",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Mt_Performative.content_union_2_type_set_of_str",
            index=10,
            number=11,
            type=9,
            cpp_type=9,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="content_union_2_type_list_of_float",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Mt_Performative.content_union_2_type_list_of_float",
            index=11,
            number=12,
            type=2,
            cpp_type=6,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="content_union_2_type_list_of_bool",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Mt_Performative.content_union_2_type_list_of_bool",
            index=12,
            number=13,
            type=8,
            cpp_type=7,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="content_union_2_type_list_of_bytes",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Mt_Performative.content_union_2_type_list_of_bytes",
            index=13,
            number=14,
            type=12,
            cpp_type=9,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="content_union_2_type_dict_of_str_int",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Mt_Performative.content_union_2_type_dict_of_str_int",
            index=14,
            number=15,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="content_union_2_type_dict_of_int_float",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Mt_Performative.content_union_2_type_dict_of_int_float",
            index=15,
            number=16,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="content_union_2_type_dict_of_bool_bytes",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Mt_Performative.content_union_2_type_dict_of_bool_bytes",
            index=16,
            number=17,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[
        _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_MT_PERFORMATIVE_CONTENTUNION1TYPEDICTOFSTRINTENTRY,
        _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_MT_PERFORMATIVE_CONTENTUNION2TYPEDICTOFSTRINTENTRY,
        _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_MT_PERFORMATIVE_CONTENTUNION2TYPEDICTOFINTFLOATENTRY,
        _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_MT_PERFORMATIVE_CONTENTUNION2TYPEDICTOFBOOLBYTESENTRY,
    ],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=4203,
    serialized_end=5695,
)

_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_O_PERFORMATIVE_CONTENTODICTSTRINTENTRY = _descriptor.Descriptor(
    name="ContentODictStrIntEntry",
    full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_O_Performative.ContentODictStrIntEntry",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="key",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_O_Performative.ContentODictStrIntEntry.key",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="value",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_O_Performative.ContentODictStrIntEntry.value",
            index=1,
            number=2,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=_b("8\001"),
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=6088,
    serialized_end=6145,
)

_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_O_PERFORMATIVE = _descriptor.Descriptor(
    name="Performative_O_Performative",
    full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_O_Performative",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="content_o_bool",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_O_Performative.content_o_bool",
            index=0,
            number=1,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="content_o_bool_is_set",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_O_Performative.content_o_bool_is_set",
            index=1,
            number=2,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="content_o_set_int",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_O_Performative.content_o_set_int",
            index=2,
            number=3,
            type=5,
            cpp_type=1,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="content_o_set_int_is_set",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_O_Performative.content_o_set_int_is_set",
            index=3,
            number=4,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="content_o_list_bytes",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_O_Performative.content_o_list_bytes",
            index=4,
            number=5,
            type=12,
            cpp_type=9,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="content_o_list_bytes_is_set",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_O_Performative.content_o_list_bytes_is_set",
            index=5,
            number=6,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="content_o_dict_str_int",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_O_Performative.content_o_dict_str_int",
            index=6,
            number=7,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="content_o_dict_str_int_is_set",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_O_Performative.content_o_dict_str_int_is_set",
            index=7,
            number=8,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[
        _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_O_PERFORMATIVE_CONTENTODICTSTRINTENTRY,
    ],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=5698,
    serialized_end=6145,
)

_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_EMPTY_CONTENTS_PERFORMATIVE = _descriptor.Descriptor(
    name="Performative_Empty_Contents_Performative",
    full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Empty_Contents_Performative",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=6147,
    serialized_end=6189,
)

_TPROTOCOLNOCTMESSAGE = _descriptor.Descriptor(
    name="TProtocolNoCtMessage",
    full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="performative_empty_contents",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.performative_empty_contents",
            index=0,
            number=5,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="performative_mt",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.performative_mt",
            index=1,
            number=6,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="performative_o",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.performative_o",
            index=2,
            number=7,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="performative_pct",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.performative_pct",
            index=3,
            number=8,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="performative_pmt",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.performative_pmt",
            index=4,
            number=9,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="performative_pt",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.performative_pt",
            index=5,
            number=10,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[
        _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PT_PERFORMATIVE,
        _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PCT_PERFORMATIVE,
        _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE,
        _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_MT_PERFORMATIVE,
        _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_O_PERFORMATIVE,
        _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_EMPTY_CONTENTS_PERFORMATIVE,
    ],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[
        _descriptor.OneofDescriptor(
            name="performative",
            full_name="aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.performative",
            index=0,
            containing_type=None,
            fields=[],
        ),
    ],
    serialized_start=57,
    serialized_end=6205,
)

_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PT_PERFORMATIVE.containing_type = (
    _TPROTOCOLNOCTMESSAGE
)
_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PCT_PERFORMATIVE.containing_type = (
    _TPROTOCOLNOCTMESSAGE
)
_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTINTBYTESENTRY.containing_type = (
    _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE
)
_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTINTINTENTRY.containing_type = (
    _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE
)
_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTINTFLOATENTRY.containing_type = (
    _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE
)
_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTINTBOOLENTRY.containing_type = (
    _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE
)
_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTINTSTRENTRY.containing_type = (
    _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE
)
_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTBOOLBYTESENTRY.containing_type = (
    _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE
)
_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTBOOLINTENTRY.containing_type = (
    _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE
)
_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTBOOLFLOATENTRY.containing_type = (
    _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE
)
_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTBOOLBOOLENTRY.containing_type = (
    _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE
)
_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTBOOLSTRENTRY.containing_type = (
    _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE
)
_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTSTRBYTESENTRY.containing_type = (
    _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE
)
_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTSTRINTENTRY.containing_type = (
    _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE
)
_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTSTRFLOATENTRY.containing_type = (
    _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE
)
_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTSTRBOOLENTRY.containing_type = (
    _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE
)
_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTSTRSTRENTRY.containing_type = (
    _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE
)
_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE.fields_by_name[
    "content_dict_int_bytes"
].message_type = (
    _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTINTBYTESENTRY
)
_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE.fields_by_name[
    "content_dict_int_int"
].message_type = (
    _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTINTINTENTRY
)
_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE.fields_by_name[
    "content_dict_int_float"
].message_type = (
    _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTINTFLOATENTRY
)
_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE.fields_by_name[
    "content_dict_int_bool"
].message_type = (
    _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTINTBOOLENTRY
)
_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE.fields_by_name[
    "content_dict_int_str"
].message_type = (
    _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTINTSTRENTRY
)
_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE.fields_by_name[
    "content_dict_bool_bytes"
].message_type = (
    _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTBOOLBYTESENTRY
)
_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE.fields_by_name[
    "content_dict_bool_int"
].message_type = (
    _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTBOOLINTENTRY
)
_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE.fields_by_name[
    "content_dict_bool_float"
].message_type = (
    _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTBOOLFLOATENTRY
)
_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE.fields_by_name[
    "content_dict_bool_bool"
].message_type = (
    _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTBOOLBOOLENTRY
)
_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE.fields_by_name[
    "content_dict_bool_str"
].message_type = (
    _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTBOOLSTRENTRY
)
_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE.fields_by_name[
    "content_dict_str_bytes"
].message_type = (
    _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTSTRBYTESENTRY
)
_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE.fields_by_name[
    "content_dict_str_int"
].message_type = (
    _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTSTRINTENTRY
)
_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE.fields_by_name[
    "content_dict_str_float"
].message_type = (
    _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTSTRFLOATENTRY
)
_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE.fields_by_name[
    "content_dict_str_bool"
].message_type = (
    _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTSTRBOOLENTRY
)
_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE.fields_by_name[
    "content_dict_str_str"
].message_type = (
    _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTSTRSTRENTRY
)
_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE.containing_type = (
    _TPROTOCOLNOCTMESSAGE
)
_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_MT_PERFORMATIVE_CONTENTUNION1TYPEDICTOFSTRINTENTRY.containing_type = (
    _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_MT_PERFORMATIVE
)
_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_MT_PERFORMATIVE_CONTENTUNION2TYPEDICTOFSTRINTENTRY.containing_type = (
    _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_MT_PERFORMATIVE
)
_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_MT_PERFORMATIVE_CONTENTUNION2TYPEDICTOFINTFLOATENTRY.containing_type = (
    _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_MT_PERFORMATIVE
)
_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_MT_PERFORMATIVE_CONTENTUNION2TYPEDICTOFBOOLBYTESENTRY.containing_type = (
    _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_MT_PERFORMATIVE
)
_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_MT_PERFORMATIVE.fields_by_name[
    "content_union_1_type_dict_of_str_int"
].message_type = _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_MT_PERFORMATIVE_CONTENTUNION1TYPEDICTOFSTRINTENTRY
_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_MT_PERFORMATIVE.fields_by_name[
    "content_union_2_type_dict_of_str_int"
].message_type = _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_MT_PERFORMATIVE_CONTENTUNION2TYPEDICTOFSTRINTENTRY
_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_MT_PERFORMATIVE.fields_by_name[
    "content_union_2_type_dict_of_int_float"
].message_type = _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_MT_PERFORMATIVE_CONTENTUNION2TYPEDICTOFINTFLOATENTRY
_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_MT_PERFORMATIVE.fields_by_name[
    "content_union_2_type_dict_of_bool_bytes"
].message_type = _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_MT_PERFORMATIVE_CONTENTUNION2TYPEDICTOFBOOLBYTESENTRY
_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_MT_PERFORMATIVE.containing_type = (
    _TPROTOCOLNOCTMESSAGE
)
_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_O_PERFORMATIVE_CONTENTODICTSTRINTENTRY.containing_type = (
    _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_O_PERFORMATIVE
)
_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_O_PERFORMATIVE.fields_by_name[
    "content_o_dict_str_int"
].message_type = (
    _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_O_PERFORMATIVE_CONTENTODICTSTRINTENTRY
)
_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_O_PERFORMATIVE.containing_type = (
    _TPROTOCOLNOCTMESSAGE
)
_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_EMPTY_CONTENTS_PERFORMATIVE.containing_type = (
    _TPROTOCOLNOCTMESSAGE
)
_TPROTOCOLNOCTMESSAGE.fields_by_name[
    "performative_empty_contents"
].message_type = _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_EMPTY_CONTENTS_PERFORMATIVE
_TPROTOCOLNOCTMESSAGE.fields_by_name[
    "performative_mt"
].message_type = _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_MT_PERFORMATIVE
_TPROTOCOLNOCTMESSAGE.fields_by_name[
    "performative_o"
].message_type = _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_O_PERFORMATIVE
_TPROTOCOLNOCTMESSAGE.fields_by_name[
    "performative_pct"
].message_type = _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PCT_PERFORMATIVE
_TPROTOCOLNOCTMESSAGE.fields_by_name[
    "performative_pmt"
].message_type = _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE
_TPROTOCOLNOCTMESSAGE.fields_by_name[
    "performative_pt"
].message_type = _TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PT_PERFORMATIVE
_TPROTOCOLNOCTMESSAGE.oneofs_by_name["performative"].fields.append(
    _TPROTOCOLNOCTMESSAGE.fields_by_name["performative_empty_contents"]
)
_TPROTOCOLNOCTMESSAGE.fields_by_name[
    "performative_empty_contents"
].containing_oneof = _TPROTOCOLNOCTMESSAGE.oneofs_by_name["performative"]
_TPROTOCOLNOCTMESSAGE.oneofs_by_name["performative"].fields.append(
    _TPROTOCOLNOCTMESSAGE.fields_by_name["performative_mt"]
)
_TPROTOCOLNOCTMESSAGE.fields_by_name[
    "performative_mt"
].containing_oneof = _TPROTOCOLNOCTMESSAGE.oneofs_by_name["performative"]
_TPROTOCOLNOCTMESSAGE.oneofs_by_name["performative"].fields.append(
    _TPROTOCOLNOCTMESSAGE.fields_by_name["performative_o"]
)
_TPROTOCOLNOCTMESSAGE.fields_by_name[
    "performative_o"
].containing_oneof = _TPROTOCOLNOCTMESSAGE.oneofs_by_name["performative"]
_TPROTOCOLNOCTMESSAGE.oneofs_by_name["performative"].fields.append(
    _TPROTOCOLNOCTMESSAGE.fields_by_name["performative_pct"]
)
_TPROTOCOLNOCTMESSAGE.fields_by_name[
    "performative_pct"
].containing_oneof = _TPROTOCOLNOCTMESSAGE.oneofs_by_name["performative"]
_TPROTOCOLNOCTMESSAGE.oneofs_by_name["performative"].fields.append(
    _TPROTOCOLNOCTMESSAGE.fields_by_name["performative_pmt"]
)
_TPROTOCOLNOCTMESSAGE.fields_by_name[
    "performative_pmt"
].containing_oneof = _TPROTOCOLNOCTMESSAGE.oneofs_by_name["performative"]
_TPROTOCOLNOCTMESSAGE.oneofs_by_name["performative"].fields.append(
    _TPROTOCOLNOCTMESSAGE.fields_by_name["performative_pt"]
)
_TPROTOCOLNOCTMESSAGE.fields_by_name[
    "performative_pt"
].containing_oneof = _TPROTOCOLNOCTMESSAGE.oneofs_by_name["performative"]
DESCRIPTOR.message_types_by_name["TProtocolNoCtMessage"] = _TPROTOCOLNOCTMESSAGE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

TProtocolNoCtMessage = _reflection.GeneratedProtocolMessageType(
    "TProtocolNoCtMessage",
    (_message.Message,),
    dict(
        Performative_Pt_Performative=_reflection.GeneratedProtocolMessageType(
            "Performative_Pt_Performative",
            (_message.Message,),
            dict(
                DESCRIPTOR=_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PT_PERFORMATIVE,
                __module__="t_protocol_no_ct_pb2"
                # @@protoc_insertion_point(class_scope:aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pt_Performative)
            ),
        ),
        Performative_Pct_Performative=_reflection.GeneratedProtocolMessageType(
            "Performative_Pct_Performative",
            (_message.Message,),
            dict(
                DESCRIPTOR=_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PCT_PERFORMATIVE,
                __module__="t_protocol_no_ct_pb2"
                # @@protoc_insertion_point(class_scope:aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pct_Performative)
            ),
        ),
        Performative_Pmt_Performative=_reflection.GeneratedProtocolMessageType(
            "Performative_Pmt_Performative",
            (_message.Message,),
            dict(
                ContentDictIntBytesEntry=_reflection.GeneratedProtocolMessageType(
                    "ContentDictIntBytesEntry",
                    (_message.Message,),
                    dict(
                        DESCRIPTOR=_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTINTBYTESENTRY,
                        __module__="t_protocol_no_ct_pb2"
                        # @@protoc_insertion_point(class_scope:aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictIntBytesEntry)
                    ),
                ),
                ContentDictIntIntEntry=_reflection.GeneratedProtocolMessageType(
                    "ContentDictIntIntEntry",
                    (_message.Message,),
                    dict(
                        DESCRIPTOR=_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTINTINTENTRY,
                        __module__="t_protocol_no_ct_pb2"
                        # @@protoc_insertion_point(class_scope:aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictIntIntEntry)
                    ),
                ),
                ContentDictIntFloatEntry=_reflection.GeneratedProtocolMessageType(
                    "ContentDictIntFloatEntry",
                    (_message.Message,),
                    dict(
                        DESCRIPTOR=_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTINTFLOATENTRY,
                        __module__="t_protocol_no_ct_pb2"
                        # @@protoc_insertion_point(class_scope:aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictIntFloatEntry)
                    ),
                ),
                ContentDictIntBoolEntry=_reflection.GeneratedProtocolMessageType(
                    "ContentDictIntBoolEntry",
                    (_message.Message,),
                    dict(
                        DESCRIPTOR=_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTINTBOOLENTRY,
                        __module__="t_protocol_no_ct_pb2"
                        # @@protoc_insertion_point(class_scope:aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictIntBoolEntry)
                    ),
                ),
                ContentDictIntStrEntry=_reflection.GeneratedProtocolMessageType(
                    "ContentDictIntStrEntry",
                    (_message.Message,),
                    dict(
                        DESCRIPTOR=_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTINTSTRENTRY,
                        __module__="t_protocol_no_ct_pb2"
                        # @@protoc_insertion_point(class_scope:aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictIntStrEntry)
                    ),
                ),
                ContentDictBoolBytesEntry=_reflection.GeneratedProtocolMessageType(
                    "ContentDictBoolBytesEntry",
                    (_message.Message,),
                    dict(
                        DESCRIPTOR=_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTBOOLBYTESENTRY,
                        __module__="t_protocol_no_ct_pb2"
                        # @@protoc_insertion_point(class_scope:aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictBoolBytesEntry)
                    ),
                ),
                ContentDictBoolIntEntry=_reflection.GeneratedProtocolMessageType(
                    "ContentDictBoolIntEntry",
                    (_message.Message,),
                    dict(
                        DESCRIPTOR=_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTBOOLINTENTRY,
                        __module__="t_protocol_no_ct_pb2"
                        # @@protoc_insertion_point(class_scope:aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictBoolIntEntry)
                    ),
                ),
                ContentDictBoolFloatEntry=_reflection.GeneratedProtocolMessageType(
                    "ContentDictBoolFloatEntry",
                    (_message.Message,),
                    dict(
                        DESCRIPTOR=_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTBOOLFLOATENTRY,
                        __module__="t_protocol_no_ct_pb2"
                        # @@protoc_insertion_point(class_scope:aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictBoolFloatEntry)
                    ),
                ),
                ContentDictBoolBoolEntry=_reflection.GeneratedProtocolMessageType(
                    "ContentDictBoolBoolEntry",
                    (_message.Message,),
                    dict(
                        DESCRIPTOR=_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTBOOLBOOLENTRY,
                        __module__="t_protocol_no_ct_pb2"
                        # @@protoc_insertion_point(class_scope:aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictBoolBoolEntry)
                    ),
                ),
                ContentDictBoolStrEntry=_reflection.GeneratedProtocolMessageType(
                    "ContentDictBoolStrEntry",
                    (_message.Message,),
                    dict(
                        DESCRIPTOR=_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTBOOLSTRENTRY,
                        __module__="t_protocol_no_ct_pb2"
                        # @@protoc_insertion_point(class_scope:aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictBoolStrEntry)
                    ),
                ),
                ContentDictStrBytesEntry=_reflection.GeneratedProtocolMessageType(
                    "ContentDictStrBytesEntry",
                    (_message.Message,),
                    dict(
                        DESCRIPTOR=_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTSTRBYTESENTRY,
                        __module__="t_protocol_no_ct_pb2"
                        # @@protoc_insertion_point(class_scope:aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictStrBytesEntry)
                    ),
                ),
                ContentDictStrIntEntry=_reflection.GeneratedProtocolMessageType(
                    "ContentDictStrIntEntry",
                    (_message.Message,),
                    dict(
                        DESCRIPTOR=_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTSTRINTENTRY,
                        __module__="t_protocol_no_ct_pb2"
                        # @@protoc_insertion_point(class_scope:aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictStrIntEntry)
                    ),
                ),
                ContentDictStrFloatEntry=_reflection.GeneratedProtocolMessageType(
                    "ContentDictStrFloatEntry",
                    (_message.Message,),
                    dict(
                        DESCRIPTOR=_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTSTRFLOATENTRY,
                        __module__="t_protocol_no_ct_pb2"
                        # @@protoc_insertion_point(class_scope:aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictStrFloatEntry)
                    ),
                ),
                ContentDictStrBoolEntry=_reflection.GeneratedProtocolMessageType(
                    "ContentDictStrBoolEntry",
                    (_message.Message,),
                    dict(
                        DESCRIPTOR=_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTSTRBOOLENTRY,
                        __module__="t_protocol_no_ct_pb2"
                        # @@protoc_insertion_point(class_scope:aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictStrBoolEntry)
                    ),
                ),
                ContentDictStrStrEntry=_reflection.GeneratedProtocolMessageType(
                    "ContentDictStrStrEntry",
                    (_message.Message,),
                    dict(
                        DESCRIPTOR=_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTSTRSTRENTRY,
                        __module__="t_protocol_no_ct_pb2"
                        # @@protoc_insertion_point(class_scope:aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictStrStrEntry)
                    ),
                ),
                DESCRIPTOR=_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE,
                __module__="t_protocol_no_ct_pb2"
                # @@protoc_insertion_point(class_scope:aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Pmt_Performative)
            ),
        ),
        Performative_Mt_Performative=_reflection.GeneratedProtocolMessageType(
            "Performative_Mt_Performative",
            (_message.Message,),
            dict(
                ContentUnion1TypeDictOfStrIntEntry=_reflection.GeneratedProtocolMessageType(
                    "ContentUnion1TypeDictOfStrIntEntry",
                    (_message.Message,),
                    dict(
                        DESCRIPTOR=_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_MT_PERFORMATIVE_CONTENTUNION1TYPEDICTOFSTRINTENTRY,
                        __module__="t_protocol_no_ct_pb2"
                        # @@protoc_insertion_point(class_scope:aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Mt_Performative.ContentUnion1TypeDictOfStrIntEntry)
                    ),
                ),
                ContentUnion2TypeDictOfStrIntEntry=_reflection.GeneratedProtocolMessageType(
                    "ContentUnion2TypeDictOfStrIntEntry",
                    (_message.Message,),
                    dict(
                        DESCRIPTOR=_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_MT_PERFORMATIVE_CONTENTUNION2TYPEDICTOFSTRINTENTRY,
                        __module__="t_protocol_no_ct_pb2"
                        # @@protoc_insertion_point(class_scope:aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Mt_Performative.ContentUnion2TypeDictOfStrIntEntry)
                    ),
                ),
                ContentUnion2TypeDictOfIntFloatEntry=_reflection.GeneratedProtocolMessageType(
                    "ContentUnion2TypeDictOfIntFloatEntry",
                    (_message.Message,),
                    dict(
                        DESCRIPTOR=_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_MT_PERFORMATIVE_CONTENTUNION2TYPEDICTOFINTFLOATENTRY,
                        __module__="t_protocol_no_ct_pb2"
                        # @@protoc_insertion_point(class_scope:aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Mt_Performative.ContentUnion2TypeDictOfIntFloatEntry)
                    ),
                ),
                ContentUnion2TypeDictOfBoolBytesEntry=_reflection.GeneratedProtocolMessageType(
                    "ContentUnion2TypeDictOfBoolBytesEntry",
                    (_message.Message,),
                    dict(
                        DESCRIPTOR=_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_MT_PERFORMATIVE_CONTENTUNION2TYPEDICTOFBOOLBYTESENTRY,
                        __module__="t_protocol_no_ct_pb2"
                        # @@protoc_insertion_point(class_scope:aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Mt_Performative.ContentUnion2TypeDictOfBoolBytesEntry)
                    ),
                ),
                DESCRIPTOR=_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_MT_PERFORMATIVE,
                __module__="t_protocol_no_ct_pb2"
                # @@protoc_insertion_point(class_scope:aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Mt_Performative)
            ),
        ),
        Performative_O_Performative=_reflection.GeneratedProtocolMessageType(
            "Performative_O_Performative",
            (_message.Message,),
            dict(
                ContentODictStrIntEntry=_reflection.GeneratedProtocolMessageType(
                    "ContentODictStrIntEntry",
                    (_message.Message,),
                    dict(
                        DESCRIPTOR=_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_O_PERFORMATIVE_CONTENTODICTSTRINTENTRY,
                        __module__="t_protocol_no_ct_pb2"
                        # @@protoc_insertion_point(class_scope:aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_O_Performative.ContentODictStrIntEntry)
                    ),
                ),
                DESCRIPTOR=_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_O_PERFORMATIVE,
                __module__="t_protocol_no_ct_pb2"
                # @@protoc_insertion_point(class_scope:aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_O_Performative)
            ),
        ),
        Performative_Empty_Contents_Performative=_reflection.GeneratedProtocolMessageType(
            "Performative_Empty_Contents_Performative",
            (_message.Message,),
            dict(
                DESCRIPTOR=_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_EMPTY_CONTENTS_PERFORMATIVE,
                __module__="t_protocol_no_ct_pb2"
                # @@protoc_insertion_point(class_scope:aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage.Performative_Empty_Contents_Performative)
            ),
        ),
        DESCRIPTOR=_TPROTOCOLNOCTMESSAGE,
        __module__="t_protocol_no_ct_pb2"
        # @@protoc_insertion_point(class_scope:aea.fetchai.t_protocol_no_ct.TProtocolNoCtMessage)
    ),
)
_sym_db.RegisterMessage(TProtocolNoCtMessage)
_sym_db.RegisterMessage(TProtocolNoCtMessage.Performative_Pt_Performative)
_sym_db.RegisterMessage(TProtocolNoCtMessage.Performative_Pct_Performative)
_sym_db.RegisterMessage(TProtocolNoCtMessage.Performative_Pmt_Performative)
_sym_db.RegisterMessage(
    TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictIntBytesEntry
)
_sym_db.RegisterMessage(
    TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictIntIntEntry
)
_sym_db.RegisterMessage(
    TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictIntFloatEntry
)
_sym_db.RegisterMessage(
    TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictIntBoolEntry
)
_sym_db.RegisterMessage(
    TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictIntStrEntry
)
_sym_db.RegisterMessage(
    TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictBoolBytesEntry
)
_sym_db.RegisterMessage(
    TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictBoolIntEntry
)
_sym_db.RegisterMessage(
    TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictBoolFloatEntry
)
_sym_db.RegisterMessage(
    TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictBoolBoolEntry
)
_sym_db.RegisterMessage(
    TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictBoolStrEntry
)
_sym_db.RegisterMessage(
    TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictStrBytesEntry
)
_sym_db.RegisterMessage(
    TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictStrIntEntry
)
_sym_db.RegisterMessage(
    TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictStrFloatEntry
)
_sym_db.RegisterMessage(
    TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictStrBoolEntry
)
_sym_db.RegisterMessage(
    TProtocolNoCtMessage.Performative_Pmt_Performative.ContentDictStrStrEntry
)
_sym_db.RegisterMessage(TProtocolNoCtMessage.Performative_Mt_Performative)
_sym_db.RegisterMessage(
    TProtocolNoCtMessage.Performative_Mt_Performative.ContentUnion1TypeDictOfStrIntEntry
)
_sym_db.RegisterMessage(
    TProtocolNoCtMessage.Performative_Mt_Performative.ContentUnion2TypeDictOfStrIntEntry
)
_sym_db.RegisterMessage(
    TProtocolNoCtMessage.Performative_Mt_Performative.ContentUnion2TypeDictOfIntFloatEntry
)
_sym_db.RegisterMessage(
    TProtocolNoCtMessage.Performative_Mt_Performative.ContentUnion2TypeDictOfBoolBytesEntry
)
_sym_db.RegisterMessage(TProtocolNoCtMessage.Performative_O_Performative)
_sym_db.RegisterMessage(
    TProtocolNoCtMessage.Performative_O_Performative.ContentODictStrIntEntry
)
_sym_db.RegisterMessage(TProtocolNoCtMessage.Performative_Empty_Contents_Performative)


_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTINTBYTESENTRY._options = (
    None
)
_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTINTINTENTRY._options = (
    None
)
_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTINTFLOATENTRY._options = (
    None
)
_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTINTBOOLENTRY._options = (
    None
)
_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTINTSTRENTRY._options = (
    None
)
_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTBOOLBYTESENTRY._options = (
    None
)
_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTBOOLINTENTRY._options = (
    None
)
_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTBOOLFLOATENTRY._options = (
    None
)
_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTBOOLBOOLENTRY._options = (
    None
)
_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTBOOLSTRENTRY._options = (
    None
)
_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTSTRBYTESENTRY._options = (
    None
)
_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTSTRINTENTRY._options = (
    None
)
_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTSTRFLOATENTRY._options = (
    None
)
_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTSTRBOOLENTRY._options = (
    None
)
_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_PMT_PERFORMATIVE_CONTENTDICTSTRSTRENTRY._options = (
    None
)
_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_MT_PERFORMATIVE_CONTENTUNION1TYPEDICTOFSTRINTENTRY._options = (
    None
)
_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_MT_PERFORMATIVE_CONTENTUNION2TYPEDICTOFSTRINTENTRY._options = (
    None
)
_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_MT_PERFORMATIVE_CONTENTUNION2TYPEDICTOFINTFLOATENTRY._options = (
    None
)
_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_MT_PERFORMATIVE_CONTENTUNION2TYPEDICTOFBOOLBYTESENTRY._options = (
    None
)
_TPROTOCOLNOCTMESSAGE_PERFORMATIVE_O_PERFORMATIVE_CONTENTODICTSTRINTENTRY._options = (
    None
)
# @@protoc_insertion_point(module_scope)
