# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2018-2019 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""This test module contains the tests for commands in aea.cli.utils.loggers module."""

from unittest import TestCase, mock

from aea.cli.utils.loggers import ColorFormatter


class ColorFormatterTestCase(TestCase):
    """Test case for ColorFormatter class."""

    def test_format_positive(self):
        """Test for format method positive result."""
        record = mock.Mock()
        record.exc_info = None
        record.levelname = "DEBUG"
        record.getMessage = mock.Mock(return_value="Message")
        color_formatter = ColorFormatter()
        color_formatter.format(record)
