# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: base.proto

import sys

_b = sys.version_info[0] < 3 and (lambda x: x) or (lambda x: x.encode("latin1"))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2

DESCRIPTOR = _descriptor.FileDescriptor(
    name="base.proto",
    package="aea",
    syntax="proto3",
    serialized_options=None,
    serialized_pb=_b(
        '\n\nbase.proto\x12\x03\x61\x65\x61\x1a\x1cgoogle/protobuf/struct.proto"\x90\x01\n\x0f\x44ialogueMessage\x12\x12\n\nmessage_id\x18\x01 \x01(\x05\x12"\n\x1a\x64ialogue_starter_reference\x18\x02 \x01(\t\x12$\n\x1c\x64ialogue_responder_reference\x18\x03 \x01(\t\x12\x0e\n\x06target\x18\x04 \x01(\x05\x12\x0f\n\x07\x63ontent\x18\x05 \x01(\x0c"o\n\x07Message\x12\'\n\x04\x62ody\x18\x01 \x01(\x0b\x32\x17.google.protobuf.StructH\x00\x12\x30\n\x10\x64ialogue_message\x18\x02 \x01(\x0b\x32\x14.aea.DialogueMessageH\x00\x42\t\n\x07message"Y\n\x08\x45nvelope\x12\n\n\x02to\x18\x01 \x01(\t\x12\x0e\n\x06sender\x18\x02 \x01(\t\x12\x13\n\x0bprotocol_id\x18\x03 \x01(\t\x12\x0f\n\x07message\x18\x04 \x01(\x0c\x12\x0b\n\x03uri\x18\x05 \x01(\tb\x06proto3'
    ),
    dependencies=[google_dot_protobuf_dot_struct__pb2.DESCRIPTOR,],
)


_DIALOGUEMESSAGE = _descriptor.Descriptor(
    name="DialogueMessage",
    full_name="aea.DialogueMessage",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="message_id",
            full_name="aea.DialogueMessage.message_id",
            index=0,
            number=1,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="dialogue_starter_reference",
            full_name="aea.DialogueMessage.dialogue_starter_reference",
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="dialogue_responder_reference",
            full_name="aea.DialogueMessage.dialogue_responder_reference",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="target",
            full_name="aea.DialogueMessage.target",
            index=3,
            number=4,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="content",
            full_name="aea.DialogueMessage.content",
            index=4,
            number=5,
            type=12,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b(""),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=50,
    serialized_end=194,
)


_MESSAGE = _descriptor.Descriptor(
    name="Message",
    full_name="aea.Message",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="body",
            full_name="aea.Message.body",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="dialogue_message",
            full_name="aea.Message.dialogue_message",
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[
        _descriptor.OneofDescriptor(
            name="message",
            full_name="aea.Message.message",
            index=0,
            containing_type=None,
            fields=[],
        ),
    ],
    serialized_start=196,
    serialized_end=307,
)


_ENVELOPE = _descriptor.Descriptor(
    name="Envelope",
    full_name="aea.Envelope",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="to",
            full_name="aea.Envelope.to",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="sender",
            full_name="aea.Envelope.sender",
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="protocol_id",
            full_name="aea.Envelope.protocol_id",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="message",
            full_name="aea.Envelope.message",
            index=3,
            number=4,
            type=12,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b(""),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="uri",
            full_name="aea.Envelope.uri",
            index=4,
            number=5,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=309,
    serialized_end=398,
)

_MESSAGE.fields_by_name[
    "body"
].message_type = google_dot_protobuf_dot_struct__pb2._STRUCT
_MESSAGE.fields_by_name["dialogue_message"].message_type = _DIALOGUEMESSAGE
_MESSAGE.oneofs_by_name["message"].fields.append(_MESSAGE.fields_by_name["body"])
_MESSAGE.fields_by_name["body"].containing_oneof = _MESSAGE.oneofs_by_name["message"]
_MESSAGE.oneofs_by_name["message"].fields.append(
    _MESSAGE.fields_by_name["dialogue_message"]
)
_MESSAGE.fields_by_name["dialogue_message"].containing_oneof = _MESSAGE.oneofs_by_name[
    "message"
]
DESCRIPTOR.message_types_by_name["DialogueMessage"] = _DIALOGUEMESSAGE
DESCRIPTOR.message_types_by_name["Message"] = _MESSAGE
DESCRIPTOR.message_types_by_name["Envelope"] = _ENVELOPE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

DialogueMessage = _reflection.GeneratedProtocolMessageType(
    "DialogueMessage",
    (_message.Message,),
    dict(
        DESCRIPTOR=_DIALOGUEMESSAGE,
        __module__="base_pb2"
        # @@protoc_insertion_point(class_scope:aea.DialogueMessage)
    ),
)
_sym_db.RegisterMessage(DialogueMessage)

Message = _reflection.GeneratedProtocolMessageType(
    "Message",
    (_message.Message,),
    dict(
        DESCRIPTOR=_MESSAGE,
        __module__="base_pb2"
        # @@protoc_insertion_point(class_scope:aea.Message)
    ),
)
_sym_db.RegisterMessage(Message)

Envelope = _reflection.GeneratedProtocolMessageType(
    "Envelope",
    (_message.Message,),
    dict(
        DESCRIPTOR=_ENVELOPE,
        __module__="base_pb2"
        # @@protoc_insertion_point(class_scope:aea.Envelope)
    ),
)
_sym_db.RegisterMessage(Envelope)


# @@protoc_insertion_point(module_scope)
